import os

import numpy as np
import numpy.typing as npt
from ovito.io import export_file, import_file
from ovito.modifiers import SelectTypeModifier


"""
I had big problems to get the ovito export_file module to do what I want, to put a numpy ndarry to a lammpstrj.
"""


def to_lammps(trjfile: str, indices_per_atom: npt.NDArray[np.float64]) -> str:
    pipeline = import_file(trjfile, sort_particles=True)

    for frame, linde in enumerate(indices_per_atom):
        data = pipeline.compute(frame)
        data.particles_.create_property("lindemann", data=linde)
        export_file(
            data,
            f"lindemann_outputfile_X{frame}.dump",
            "lammps/dump",
            columns=[
                "Particle Identifier",
                "Particle Type",
                "Position.X",
                "Position.Y",
                "Position.Z",
                "lindemann",
            ],
            multiple_frames=True,
        )

    """
    This is section is weird and very hacky, I dont like it. Does someone have a idea how to do this in a better way?
    First I save the files with the ovito export_file, then I save them in a file, finally I remove the files... I know...
    """

    filenames = [f"lindemann_outputfile_X{frame}.dump" for frame in range(len(indices_per_atom))]

    file_name = "lindemann_per_atom.lammpstrj"
    with open(file_name, "w") as outfile:
        for fname in filenames:
            with open(fname) as infile:
                outfile.write(infile.read())
    for frame in range(len(indices_per_atom)):
        os.remove(f"lindemann_outputfile_X{frame}.dump")
    return "saved trajectory as lindemann_per_atom.lammpstrj"



# type: ignore[attr-defined]
"""lindemann is a Python package to calculate the Lindemann index of a LAMMPS trajectory, as well as the progression of the Lindemann index per frame or per atom and frame of temperature ramps for phase transition analysis."""


try:
    from importlib.metadata import version, PackageNotFoundError
except ImportError:  # pragma: no cover
    from importlib_metadata import version, PackageNotFoundError


try:
    __version__ = version(__name__)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"



import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import numpy.typing as npt

mpl.use("Agg")


def lindemann_vs_frames(indices: npt.NDArray[np.float64]) -> str:
    plt.figure(1)
    plt.title("Lindemann index per frame")
    plt.xlabel("Frames")
    plt.ylabel("Lindemann index")
    plt.plot(np.arange(0, len(indices)), indices, "+")
    plt.tight_layout()
    # plt.show()
    plt.savefig("lindemann_per_frame.pdf")
    return "lindemann_per_frame.pdf"



import os
from typing import Optional

import numpy as np
import numpy.typing as npt
from ovito.io import import_file
from ovito.modifiers import SelectTypeModifier


def frames(trjfile: str, nframes: Optional[int] = None) -> npt.NDArray[np.float64]:

    """
    Get the frames from the lammps trajectory using ovito pipeline and import_file function.
    It returns frames and the number of frames to use for calculating the Lindemann Index.
    """

    if not os.path.exists(trjfile):
        raise RuntimeError(f"Error: file {trjfile} not found!")

    pipeline = import_file(trjfile, sort_particles=True)
    num_frame = pipeline.source.num_frames
    pipeline.modifiers.append(
        SelectTypeModifier(operate_on="particles", property="Particle Type", types={1, 2, 3})
    )
    data = pipeline.compute()
    num_particle = data.particles.count

    # If no argument is given use all frames
    if nframes is None:
        nframes = num_frame
    # make sure nobody puts more frames then exists
    assert num_frame >= nframes

    # initialise array, could be problematic for big clusters and a lot of frames
    position = np.zeros((nframes, num_particle, 3))

    for frame in range(nframes):
        data = pipeline.compute(frame)
        position[frame, :, :] = np.array(data.particles["Position"])
    frames = position

    return frames



# type: ignore[attr-defined]
# https://github.com/numba/numba/issues/4713
import random
import time
from enum import Enum
from multiprocessing import Pool
from os import sched_getaffinity
from pathlib import Path
from typing import List, Optional

import numpy as np
import typer
from rich.console import Console

from lindemann import __version__
from lindemann.index import mem_use, per_atoms, per_frames, per_trj
from lindemann.trajectory import plt_plot, read, save

app = typer.Typer(
    name="lindemann",
    help="""lindemann is a Python package to calculate the Lindemann index of a LAMMPS trajectory,
    as well as the progression of the Lindemann index per frame or per atom and frame of 
    temperature ramps for phase transition analysis.""",
    add_completion=False,
)
console = Console()


def version_callback(value: bool):
    """Prints the version of the package."""
    if value:
        console.print(f"[magenta]lindemann[/] version: [bold blue]{__version__}[/]")
        raise typer.Exit()


@app.command()
def main(
    trjfile: List[Path] = typer.Argument(
        ...,
        help="The trajectory file(s). If no other option is selected, the lindemann index is calculated for the trajectory. \
            Equivalent to the -t option. If you pass more than one trajectory they will be calculated in parallel. \
                Only works with no flag or -t flag.",
    ),
    trj: bool = typer.Option(
        False, "-t", help="Calculates the Lindemann-Index for the Trajectory file(s)",
    ),
    frames: bool = typer.Option(
        False, "-f", help="Calculates the Lindemann-Index for each frame."
    ),
    atoms: bool = typer.Option(
        False, "-a", help="Calculates the Lindemann-Index for each atom for each frame.",
    ),
    plot: bool = typer.Option(False, "-p", help="Returns a plot Lindemann-Index vs. Frame."),
    lammpstrj: bool = typer.Option(
        False,
        "-l",
        help="Saves the individual Lindemann-Index of each Atom in a lammpstrj, so it can be viewed in Ovito.",
    ),
    version: bool = typer.Option(
        None,
        "-v",
        "--version",
        callback=version_callback,
        is_eager=True,
        help="Prints the version of the lindemann package.",
    ),
    timeit: bool = typer.Option(
        False, "-ti", "-timeit", help="Uses timeit module to show running time",
    ),
    mem_useage: bool = typer.Option(
        False,
        "-m",
        "-mem_use",
        help="Calculates the memory use. Run it before you use any of the cli functionality despite the -t flag",
    ),
):

    """
    lindemann is a Python package to calculate the Lindemann index of a LAMMPS trajectory, as well
    as the progression of the Lindemann index per frame or per atom and frame of temperature ramps
    for phase transition analysis.
    """
    # frames = read.frames(trjfile)
    # frames = lindemann.trajectory.read.frames(trjfile)
    start = time.time()

    n_cores = len(sched_getaffinity(0))
    len_trjfiles = len(trjfile)

    if len_trjfiles > n_cores:
        n_cores = n_cores
    else:
        n_cores = len_trjfiles

    if len(trjfile) == 1:
        single_process = True
        tjr_frames = read.frames(str(trjfile[0]))
    else:
        single_process = False
        trjfile = [str(trjf) for trjf in trjfile]
        tjr_frames = [read.frames(tf) for tf in trjfile]

    if trj and single_process:

        console.print(
            f"[magenta]lindemann index for the Trajectory:[/] [bold blue]{per_trj.calculate(tjr_frames)}[/]"
        )
        raise typer.Exit()
    elif trj and not single_process:
        with Pool(n_cores) as p:
            console.print(f"Using {n_cores} cores")
            res = p.map(per_trj.calculate, tjr_frames)
            console.print(res)
        raise typer.Exit()

    elif frames and single_process:
        my_file_name = False
        if my_file_name:
            print("not implemented")
        else:
            filename = "lindemann_per_frame.txt"
        np.savetxt(filename, per_frames.calculate(tjr_frames))
        console.print(f"[magenta]lindemann index per frame saved as:[/] [bold blue]{filename}[/]")
        raise typer.Exit()
    elif frames and not single_process:
        print("multiprocessing is implemented only for the -t flag")
        raise typer.Exit()

    elif atoms and single_process:
        filename = "lindemann_per_atom.txt"
        np.savetxt(filename, per_atoms.calculate(tjr_frames))
        console.print(f"[magenta]lindemann index per atoms saved as:[/] [bold blue]{filename}[/]")
        raise typer.Exit()
    elif atoms and not single_process:
        print("multiprocessing is implemented only for the -t flag")
        raise typer.Exit()

    elif plot and single_process:
        indices = per_frames.calculate(tjr_frames)

        console.print(
            f"[magenta]Saved file as:[/] [bold blue]{plt_plot.lindemann_vs_frames(indices)}[/]"
        )
        raise typer.Exit()
    elif plot and not single_process:
        print("multiprocessing is implemented only for the -t flag")
        raise typer.Exit()

    elif lammpstrj and single_process:
        indices_per_atom = per_atoms.calculate(tjr_frames)

        console.print(f"[magenta]{save.to_lammps(trjfile,indices_per_atom)}[/]")
        raise typer.Exit()
    elif lammpstrj and not single_process:
        print("multiprocessing is implemented only for the -t flag")
        raise typer.Exit()

    elif timeit and single_process:

        start = time.time()
        linde_for_time = per_trj.calculate(tjr_frames)
        time_diff = time.time() - start

        console.print(
            f"[magenta]lindemann index for the Trajectory:[/] [bold blue]{linde_for_time}[/] \n[magenta]Runtime:[/] [bold green]{time_diff}[/]"
        )
        raise typer.Exit()
    elif timeit and not single_process:
        print("multiprocessing is implemented only for the -t flag")
        raise typer.Exit()

    elif mem_useage and single_process:

        mem_use_in_gb = mem_use.in_gb(tjr_frames)

        console.print(f"[magenta]memory use:[/] [bold blue]{mem_use_in_gb}[/]")
        raise typer.Exit()
    elif mem_useage and not single_process:
        print("multiprocessing is implemented only for the -t flag")
        raise typer.Exit()
    else:
        if single_process:
            console.print(
                f"[magenta]lindemann index for the Trajectory:[/] [bold blue]{per_trj.calculate(tjr_frames)}[/]"
            )
            raise typer.Exit()
        elif not single_process:
            with Pool(n_cores) as p:
                console.print(f"Using {n_cores} cores")
                res = p.map(per_trj.calculate, tjr_frames)
                console.print(res)
            raise typer.Exit()



from typing import Any

import numba as nb
import numpy as np
import numpy.typing as npt

# No typing for jit functions https://github.com/numba/numba/issues/7424


@nb.njit(fastmath=True, error_model="numpy")  # type: ignore
def lindemann_per_atom(frames: npt.NDArray[np.float64]) -> Any:

    """Calculates the lindemann index for """
    dt = frames.dtype
    natoms = len(frames[0])
    nframes = len(frames)
    array_mean = np.zeros((natoms, natoms), dtype=dt)
    array_var = np.zeros((natoms, natoms), dtype=dt)
    # array_distance = np.zeros((natoms, natoms),dtype=dt)
    iframe = dt.type(1)
    for coords in frames:

        # here we do someting similar to scipy's spatial.distance.pdist scipy.spatial.distance.pdist
        n, p = coords.shape
        array_distance = np.zeros((n, n), dtype=dt)
        for i in range(n):
            for j in range(i + 1, n):
                d = dt.type(0.0)
                for k in range(p):
                    d += (coords[i, k] - coords[j, k]) ** dt.type(2)
                array_distance[i, j] = np.sqrt(d)
        array_distance += array_distance.T

        #################################################################################
        # update mean and var arrays based on Welford algorithm suggested by Donald Knuth
        #################################################################################
        for i in range(natoms):
            for j in range(i + 1, natoms):
                xn = array_distance[i, j]
                mean = array_mean[i, j]
                var = array_var[i, j]
                delta = xn - mean
                array_mean[i, j] = mean + delta / iframe
                array_var[i, j] = var + delta * (xn - array_mean[i, j])
        iframe += 1.0
        if iframe > nframes:
            break

    for i in range(natoms):
        for j in range(i + 1, natoms):
            array_mean[j, i] = array_mean[i, j]
            array_var[j, i] = array_var[i, j]

    lindemann_indices = np.divide(np.sqrt(np.divide(array_var, nframes)), array_mean)
    return lindemann_indices


def calculate(frames: npt.NDArray[np.float64]) -> float:
    """
    Small helper function, since numba has not implemented the np.nanmean with axis parameter 
    I cant implemnet this in the jit function for now.
    """

    return np.mean(np.nanmean(lindemann_per_atom(frames), axis=1))  # type: ignore[no-any-return, no-untyped-call]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lindemann', 'lindemann.index', 'lindemann.trajectory']

package_data = \
{'': ['*']}

install_requires = \
['icc-rt>=2020.0.133,<2021.0.0',
 'matplotlib>=3.2.2,<4.0.0',
 'numba>=0.52',
 'numpy>=1.18,<1.22',
 'ovito>=3.1.1,<4.0.0',
 'pathlib>=1.0.1,<2.0.0',
 'pre-commit>=2.6.0,<3.0.0',
 'rich>=3.3.2,<11.0.0',
 'typer[all]>=0.3.0,<0.4.0']

extras_require = \
{':python_full_version >= "3.7.3" and python_version < "3.9"': ['importlib_metadata>=1.6,<5.0']}

entry_points = \
{'console_scripts': ['lindemann = lindemann.main:app']}

setup_kwargs = {
    'name': 'lindemann',
    'version': '0.4.1',
    'description': 'lindemann is a python package to calculate the Lindemann index  of a lammps trajectory as well as the progression of the Lindemann index per frame of temperature ramps  for phase transition analysis.',
    'long_description': '# lindemann\n\n<div align="center">\n\n[![Build status](https://github.com/N720720/lindemann/workflows/build/badge.svg?branch=master&event=push)](https://github.com/N720720/lindemann/actions?query=workflow%3Abuild)\n[![Python Version](https://img.shields.io/pypi/pyversions/lindemann.svg)](https://pypi.org/project/lindemann/)\n[![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/N720720/lindemann/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot)\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Security: bandit](https://img.shields.io/badge/security-bandit-green.svg)](https://github.com/PyCQA/bandit)  \n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/N720720/lindemann/blob/master/.pre-commit-config.yaml)\n[![Semantic Versions](https://img.shields.io/badge/%F0%9F%9A%80-semantic%20versions-informational.svg)](https://github.com/N720720/lindemann/releases)\n[![License](https://img.shields.io/pypi/l/lindemann)](https://pypi/l/lindemann/)\n\n![](images/459_Atoms_brass.gif)\n\nlindemann is a Python package to calculate the Lindemann index of a LAMMPS trajectory, as well as the progression of the Lindemann index per frame or per atom and frame of temperature ramps for phase transition analysis.\n</div>\n\n## Installation\n\nIt requires python 3.7\n\n```bash\npip install lindemann\n```\n\nor install with `Poetry`\n\n```bash\npoetry add lindemann\n```\n\n## Usage\n\n```console\nlindemann [OPTIONS] TRJFILE\n```\n\nor if installed with `Poetry`:\n\n```bash\npoetry run [OPTIONS] TRJFILE\n```\n\n**Precautions**:\n\nMake sure you have enough memory available before you run any flags despite the -t flag. You can check the memory use by using the `-m` or the `--mem_use` Option. The high memory use will be fixed as soon as the numpy.nanmean() function with axis is available for numba.\n\n**Multiprocessing**:\n\nIf you don\'t have a hpc environment available to distribute the workload, I added multiprocessing to parallize the tasks if you are using your local machine. Just add more than one filename with the -t flag. Currently multiprocessing is only implemented for the -t flag, due to the memory issues mentioned above.\n\n**Arguments**:\n\n* `TRJFILE...`: The trajectory file(s). If no other option is selected, the lindemann index is calculated for the trajectory. Equivalent to the -t option. If you pass more than one trajectory they will be calculated in parallel. Only works with no flag or -t flag.   [required]\n\n**Options**:\n\n* `-t`: Calculates the Lindemann-Index for the Trajectory file(s)  [default: False]\n* `-f`: Calculates the Lindemann-Index for each frame.  [default: False]\n* `-a`: Calculates the Lindemann-Index for each atom for each frame.  [default: False]\n* `-p`: Returns a plot Lindemann-Index vs. Frame.  [default: False]\n* `-l`: Saves the individual Lindemann-Index of each Atom in a lammpstrj, so it can be viewed in Ovito.  [default: False]\n* `-v, --version`: Prints the version of the lindemann package.\n* `-ti, -timeit`: Uses timeit module to show running time  [default: False]\n* `-m, -mem_use`: Calculates the memory use. Run it before you use any of the cli functionality despite the -t flag  [default: False]\n* `--help`: Show this message and exit.\n\n## Demo\n\nBasic usage to calculate the Lindemann Index:\n\n![](images/linde_t.gif)\n\nThe package has a plotting feature. It will show the a plot Lindemann index vs. the frames. If the trajectory file is a temperature ramp, it is possible to determine the phase transition.\n\n![](images/linde_p_new.gif)\n\nUsage of the of the lammpstrj file output feature to save the progression for each atom per frame into a LAMMPS trajectory file. Afterwards the trajectory can be viewed with ovito for example, here the Lindemann progression was used for the OVITO color coding feature.\n\n![](images/demo_lammps_ovito.gif)\n\n## Motivation\n\nIn my research I have investigated simulated annealing of brass nanoparticles. I had to determine the melting point of many different cluster sizes and zinc compositions and I have chosen the Lindemann index for this purpose. I wanted a cli tool that could be easily used with MPI due to the amount of calculations. I also wanted to get to know numba better and so I combined them in this project. numba does not really like pythonic code and can handle verbose code better.\n\n[`ybyygu`](https://github.com/ybyygu/lindemann-index) had already worked out a nice solution in cython with the Welford algorithm, which was found to be easy to convert into numba. [`whashi44`](https://github.com/whashi44/lindemann) in turn had presented a great Numpy version with a GUI, his version convinced me to use ovito for read write tasks of LAMMPS trajectories.\n\nFor the determination of the melting point of nanoparticles you can often find plots in the literature which follow the following scheme. A temperature range is defined where the phase transition is likely to take place. Then step lengths for the temperature are defined and then a trajectory is calculated for the respective temperature. If a phase transition occurs, this can be recognized by a jump of the lindemann index as shown in figure 1\n\n![](images/like_in_literature.png)\n\n*Figure 1.*\n\nI realized that the development of the Lindemann index can be observed across the trajectory. The Welford algorithm allows a live calculation of the mean std, which means that the development of the Lindemann index can be observed as the simulations run. Since I worked with heat ramps for simulated annaeling, I was interested in the development of the Lindemann index across the frames of these trajectories.\n\nThis could be realized by a modification of the Welford algorithm. With this method you can observe the change of the Lindemann index per frame. Here, too, a characteristic change of the Lindemann index occurs, with which the phase transition can also be determined. The development of the Lindemann index over the frames can then be visualized as a plot for the phase transition determination. If the temperature is plotted against the Lindemann index, the temperature can also be read directly for the phase transition, as seen in figure 2.\n\n![](images/new_way_linde.png)\n\n*Figure 2: Note the noise of the temperature was smoothed with help of [numpy.convolve](https://numpy.org/doc/stable/reference/generated/numpy.convolve.html) with a box size of 40.*\n\nWith a further modification of the Welford algorithm you can also see the contribution of each atom to the lindemann index for each frame. This was especially interesting for me because atomic properties can be easily visualized with tools like OVITO. This gives one a detailed view of the phase transition. This is especially interesting for the investigation of nanoparticles during the phase transition, as they often start melting from the surface. This is where the method in combination with OVITO offers a good starting point for further investigations.\n\nFurthermore, computational costs can be reduced by using one heat ramp instead of many trajectories with discrete temperatures. As an example, see figures 1 and 2. In figure 1, 16 trajectories with different temperatures were chosen to determine the phase transition. The step width was reduced near the phase transition for a better determination of the phase transition. Each trajectory has 50k steps, that is 800k in total, whereas in figure 2 the heatramp has only 200k steps.\n\n## Background\n\nA key problem with the measurement of the melting point of nanoparticles is that with decreasing size of a given nanoparticle the phase transition, defined as the temperature of a sudden change in the enthalpy, becomes less pronounced. This is caused by the surface effect: for a given cluster the surface area is larger compared to a bulk structure of the same size. Melting does not take place all at once, but is a longer melt transition and no longer really a melting point.\n\nThe Lindemann index, stated in the following equation presents a solution for this problem. It describes the root-mean-square (rms) fluctuation of the bonds or interatomic distance in the system over time (or temperature, if the temperature of the system changes as the simulation progresses). The Lindemann index is a more robust method to determine the melting point of nanoparticles, as opposed to enthalpy. Accordingly the Lindemann index is often considered, when the melting point of nano-particles is of interest. The index is defined as, \n\n<a href="https://www.codecogs.com/eqnedit.php?latex={\\langle&space;q_{i}&space;\\rangle_{\\text{atoms}}={\\frac&space;{1}{N(N-1)}}\\sum&space;_{j\\neq&space;i}{\\frac&space;{\\sqrt&space;{\\langle&space;r_{ij}^{2}\\rangle&space;-\\langle&space;r_{ij}\\rangle&space;^{2}}}{\\langle&space;r_{ij}\\rangle&space;}}}&space;~." target="_blank"><img src="https://latex.codecogs.com/gif.latex?{\\langle&space;q_{i}&space;\\rangle_{\\text{atoms}}={\\frac&space;{1}{N(N-1)}}\\sum&space;_{j\\neq&space;i}{\\frac&space;{\\sqrt&space;{\\langle&space;r_{ij}^{2}\\rangle&space;-\\langle&space;r_{ij}\\rangle&space;^{2}}}{\\langle&space;r_{ij}\\rangle&space;}}}&space;~." title="{\\langle q_{i} \\rangle_{\\text{atoms}}={\\frac {1}{N(N-1)}}\\sum _{j\\neq i}{\\frac {\\sqrt {\\langle r_{ij}^{2}\\rangle -\\langle r_{ij}\\rangle ^{2}}}{\\langle r_{ij}\\rangle }}} ~." /></a>\n\nwhere <a href="https://www.codecogs.com/eqnedit.php?latex=\\inline&space;\\fn_phv&space;\\large&space;N" target="_blank"><img src="https://latex.codecogs.com/gif.latex?\\inline&space;\\fn_phv&space;\\large&space;N" title="\\large N" /></a> is the number of atoms in the nano particle. <a href="https://www.codecogs.com/eqnedit.php?latex=\\inline&space;\\fn_phv&space;\\large&space;r_{ij}" target="_blank"><img src="https://latex.codecogs.com/gif.latex?\\inline&space;\\fn_phv&space;\\large&space;r_{ij}" title="\\large r_{ij}" /></a> is the distance between atom i and atom j. The brackets <a href="https://www.codecogs.com/eqnedit.php?latex=\\inline&space;\\fn_phv&space;\\large&space;\\langle~\\rangle" target="_blank"><img src="https://latex.codecogs.com/gif.latex?\\inline&space;\\fn_phv&space;\\large&space;\\langle~\\rangle" title="\\large \\langle~\\rangle" /></a> representing a time or temperature average. The rms of the bond fluctuation is considerable lower for a solid than for a liquid, due to restrained degrees of freedom. In a solid, the atoms hold on to their position and only fluctuate around their equilibrium positions. During the melting process the atoms become more mobile and are able to leave their original position. The translation movement of atoms is magnitudes larger than that of the bond fluctuations of a solid. As a result, the Lindemann index rises dramatically at the melting point and therefore gives a suitable observable to determine the transition phase. In effect, the Lindemann index measures a sort of average difussion coefficient for the atoms in the system.\n\nA key problem with much of the literature regarding the Lindemann index, is that there is a uncertainty of where to define the phase transition within a Lindemann plot. On the grounds that the melting point is a macroscopic definition for bulk structures. But here nanoparticles differ: because of their relative small size, compared to bulk structures, melting can occur fist on their relative large surface, compared to the volume they obtain and followed by the melting of core of the particle. Therefore a temperature range, rather then a melting point, is observed, as stated by Neyts in their work.\n\n## 🛡 License\n\n[![License](https://img.shields.io/pypi/l/lindemann)](https://pypi/l/lindemann/)\n\nThis project is licensed under the terms of the `GPLv3` license. See [LICENSE](https://github.com/N720720/lindemann/blob/master/LICENSE) for more details.\n\n## 📃 Citation\n\n```\n@misc{lindemann,\n  author = {Sebastian Thurm},\n  title = {lindemann is a python package to calculate the Lindemann index  of a lammps trajectory as well as the progression of the Lindemann index per frame of temperature ramps  for phase transition analysis.},\n  year = {2020},\n  publisher = {GitHub},\n  journal = {GitHub repository},\n  howpublished = {\\url{https://github.com/N720720/lindemann}}\n}\n```\n\n## Credits\n\nThe Lindemann index is introduced in the following paper,\\\nF. A. Lindemann, *Zeitschrift für Phys.* **1910**, *11*, 609–612.\\\nThis package is based on the work from [`ybyygu`](https://github.com/ybyygu/lindemann-index)\nand [`whashi44`](https://github.com/whashi44/lindemann) on calculating the Lindemann index.\\\nThe visualisations in this Readme are made with [`Ovito`](https://www.ovito.org/).\\\nA. Stukowski, *Model. Simul. Mater. Sci. Eng.* **2010**, *18*, 15012. [`link`](https://iopscience.iop.org/article/10.1088/0965-0393/18/1/015012).\\\nThis project was generated with [`python-package-template`](https://github.com/TezRomacH/python-package-template).\\\nThe Welford Algorith can be found in:\\\nDonald E. Knuth ,*The art of computer programming*, volume 2 (3rd ed.): seminumerical algorithms, Addison-Wesley Longman Publishing Co, **1997**, 232.\n',
    'author': 'N720720',
    'author_email': 's.thurm@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/N720720/lindemann',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.3,<3.11',
}


setup(**setup_kwargs)
